/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.gui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.plaf.BorderUIResource;
import javax.swing.text.BadLocationException;
import javax.swing.text.JTextComponent;

import org.w3c.dom.Attr;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import ch.cern.lhcb.xmleditor.specific.XmlDisplay;
import ch.cern.lhcb.xmleditor.undo.UndoableAttributeEdition;
import ch.cern.lhcb.xmleditor.xerces.ElementImpl;
import ch.cern.lhcb.xmleditor.xerces.TreeElement;

/**
 * this class implements an editor pane. This pane changes depending on the kind
 * of node to edit.
 * 
 * @author Sebastien Ponce
 */
public class EditorPane extends JPanel {

    /**
     * the text on remove popup menu item
     */
    protected final static String REMOVE = "remove attribute";

    /**
     * a pointer on the TreeView object to which this editor panel is associated
     */
    private TreeView treeView;

    /**
     * the panel where the editor is displayed
     */
    private JPanel panel;

    /**
     * the layout of panel
     */
    private GridBagLayout layout;

    /**
     * the node currently displayed
     */
    private TreeElement node;

    /**
     * This constructor creates a default (empty) panel to contain the node
     * edition facilities. It also initiates the layout manager of this panel
     */
    public EditorPane(TreeView treeView) {
        super();
        this.treeView = treeView;
        // creates an innerPanel
        panel = new JPanel();
        // sets its Layout
        layout = new GridBagLayout();
        panel.setLayout(layout);
        // adds it to the main panel
        this.add(panel, BorderLayout.NORTH);
        // display nothing
        displayNode(null);
    }

    /**
     * creates subpanels in the editorPane to display the attributes of a node
     * 
     * @param node the node we wnt to display the attributes
     * @param attrListener a listener for popup menus
     */
    private void displayAttributes(TreeElement node,
            AttributeLabelListener attrListener) {
        // retrieves the attributes
        NamedNodeMap nodeAttributes = node.getAttributes();
        // if there are none, return
        if (nodeAttributes == null) { return; }
        // tries to retrieve the XmlDisplay for the node to be displayed
        XmlDisplay xmlDisplay = XmlTree.getXmlDisplayFromNode(node);
        // for each attribute
        for (int j = 0; j < nodeAttributes.getLength(); j++) {
            TreeElement attribute = (TreeElement) nodeAttributes.item(j);
            // if we have an XmlDisplay, use it to know what to display
            if (xmlDisplay != null) {
                // if needed, displays the field
                if (xmlDisplay.isAttributeDisplayed(node.getNodeName(),
                        attribute.getNodeName())) {
                    displayAttribute(attribute, xmlDisplay
                            .getAttributePossibleValues(node.getNodeName(),
                                    attribute.getNodeName()), attrListener,
                            xmlDisplay.isAttributeEditable(node.getNodeName(),
                                    attribute.getNodeName()));
                }
            } else {
                // else displays every attribute without specifying
                // any possible value and as editable
                displayAttribute(attribute, null, attrListener, true);
            }
        }
    }

    /**
     * displays a single attribute in the editorPane the way it is displayed
     * depends on its possibleValues and whether it is editable or not
     * 
     * @param attribute the attribute to display
     * @param possibleValues the values that the attribute could take or null if
     *        it can take any value
     * @param attrListener a listener for popup menus
     * @param editable tells whether this attribute value could be changed
     */
    private void displayAttribute(TreeElement attribute,
            String[] possibleValues, AttributeLabelListener attrListener,
            boolean editable) {
        if (possibleValues == null) {
            // if no special possible values provided,
            // displays a string attribute
            displayStringAttribute(attribute, attribute.getNodeName(),
                    attrListener, false, editable);
        } else {
            // else displays a list attribute with a comboBox
            // of all possible values
            displayListAttribute(attribute, attribute.getNodeName(),
                    possibleValues, attrListener, editable);
        }
    }

    /**
     * creates subpanels in the editorPane to display the children of a node
     * that the user wants to display
     * 
     * @param node the node we want to display the children
     * @param attrListener a listener for popup menus
     */
    private void displayChildren(TreeElement node,
            AttributeLabelListener attrListener) {
        // retrieves the children of the node
        NodeList nodeList = node.getChildNodes();
        // for each child
        for (int i = 0; i < nodeList.getLength(); i++) {
            // retrieves the child of the node
            TreeElement child = (TreeElement) nodeList.item(i);
            // checks the type of the node
            // TEXT_NODE means we deal with the value of node,
            // ELEMENT_NODE means we deal with a subelement
            switch (child.getNodeType()) {
            case Node.ELEMENT_NODE:
                // displays node
                XmlDisplay xmlDisplay = XmlTree.getXmlDisplayFromNode(node);
                // if a display manager exists, use it to know whether to
                // display the node or not.
                // Else, don't display it as an attribute
                boolean display = false;
                if (xmlDisplay != null) {
                    display = xmlDisplay.isSubElementDisplayedAsAttr(node
                            .getNodeName(), child.getNodeName());
                }
                if (display) {
                    displayStringAttribute((TreeElement) child.getFirstChild(),
                            child.getNodeName(), attrListener, true, true);
                }
                break;
            case Node.TEXT_NODE:
                if (treeView.isValueDisplayedAsAttribute()) {
                    this.displayStringAttribute(child, "Value", attrListener,
                            true, true);
                }
                break;
            default:
                break;
            }
        }
    }

    /**
     * displays a string attribute in the editorPane
     * 
     * @param node the attribute node
     * @param name the name of the attribute
     * @param attrListener a listener for popup menus
     * @param multiline tell whether the string could be multiline or not
     * @param editable tell whether the string is editable or not
     */
    private void displayStringAttribute(TreeElement node, String name,
            AttributeLabelListener attrListener, boolean multiline,
            boolean editable) {
        // gets the value to display
        String value;
        if (node != null) {
            value = node.getNodeValue();
        } else {
            value = "";
        }
        // builds the appropriate text component
        JTextComponent textComponent;
        if (multiline) {
            JTextArea textArea = new JTextArea(value);
            textArea.setLineWrap(false);
            textComponent = textArea;
        } else {
            textComponent = new JTextField(value);
        }
        textComponent.setEditable(editable);
        textComponent
                .setBorder(BorderUIResource.getBlackLineBorderUIResource());
        textComponent.setMargin(new Insets(0, 3, 0, 3));

        if (editable) {
            // takes a change of value into account
            textComponent.addFocusListener(new StringFocusListener(node));
            textComponent.getDocument().addDocumentListener(
                    new StringNodeValueListener(node));
            textComponent.getDocument().addUndoableEditListener(
                    treeView.getEditor().getUndoManager());
        }
        // displays
        this.achieveAttributeDisplay(name, textComponent, attrListener);
    }

    /**
     * does the actual display and layout for an attribute
     * 
     * @param name the name of the attribute
     * @param value a component displaying the value of the attribute
     * @param attrListener a listener for popup menus
     */
    private void achieveAttributeDisplay(String name, JComponent value,
            AttributeLabelListener attrListener) {
        // creates a label for displaying the name and the icon
        javax.swing.Icon icon = treeView.getEditor().getIconsManager().getIcon(
                name);
        JLabel label = new JLabel(name, icon, SwingConstants.LEFT);
        // adds a mouse listener to make a popupmenu appear in case
        // of right click
        label.addMouseListener(attrListener);
        // sets some constraints
        GridBagConstraints c = new GridBagConstraints();
        c.weightx = 1;
        c.weighty = 0;
        c.gridx = 0;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.WEST;
        c.insets = new Insets(0, 5, 0, 0);
        // adds the label
        layout.setConstraints(label, c);
        panel.add(label);
        // new constraints
        c.weightx = 0;
        c.gridx = 1;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.EAST;
        c.insets = new Insets(0, 10, 0, 0);
        // adds the value
        layout.setConstraints(value, c);
        panel.add(value);
    }

    /**
     * displays a list attribute in the editorPane
     * 
     * @param node the attribute node
     * @param name the name of the attribute
     * @param values the list of possible values for this attribute
     * @param attrListener a listener for popup menus
     * @param editable tell whether the attribute is editable or not
     */
    private void displayListAttribute(TreeElement node, String name,
            String[] values, AttributeLabelListener attrListener,
            boolean editable) {
        // uses a ComboBox to edit the value
        // creates the comboBox
        JComboBox comboBox = new JComboBox(values);
        // sets the default value
        int index = 0;
        for (index = 0; index < values.length; index++) {
            if (values[index].equals(node.getNodeValue())) {
                break;
            }
        }
        if (index < values.length) {
            comboBox.setSelectedIndex(index);
        }
        comboBox.setEnabled(editable);
        if (editable) {
            // take a change of value into account
            comboBox.addActionListener(new ListNodeValueListener(node));
        }
        // displays
        this.achieveAttributeDisplay(name, comboBox, attrListener);
    }

    /**
     * builds an editor pane for a given node
     * 
     * @param node the node we want to display
     */
    public void displayNode(TreeElement node) {
        // keep a pointer on the displayed node
        this.node = node;
        // changes the title
        if (node == null) {
            this.setBorder(BorderFactory.createTitledBorder("No Node"));
        } else {
            this
                    .setBorder(BorderFactory.createTitledBorder(node
                            .getNodeName()));
        }
        // displays it
        this.redisplay();
    }

    /**
     * builds a title bar in the editor Pane
     */
    private void displayTitles() {
        // sets some constraints
        GridBagConstraints c = new GridBagConstraints();
        c.weightx = 1;
        c.weighty = 0;
        c.gridx = 0;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.WEST;
        c.insets = new Insets(0, 5, 5, 0);
        // adds the label
        JLabel label = new JLabel("Attributes");
        Font font = new Font("dialog", Font.PLAIN, 16);
        label.setFont(font);
        layout.setConstraints(label, c);
        panel.add(label);
        // new constraints
        c.weightx = 0;
        c.gridx = 1;
        c.anchor = GridBagConstraints.CENTER;
        c.insets = new Insets(0, 10, 5, 0);
        // adds the value
        JLabel value = new JLabel("Values");
        value.setFont(font);
        layout.setConstraints(value, c);
        panel.add(value);
    }

    /**
     * rebuilds the editor pane
     */
    public void redisplay() {
        // cleans up panel
        panel.removeAll();
        // changes the title
        if (node == null) {
            this.setBorder(BorderFactory.createTitledBorder("No Node"));
            return;
        } else {
            this
                    .setBorder(BorderFactory.createTitledBorder(node
                            .getNodeName()));
        }
        // displays a title bar
        displayTitles();
        // creates a listener for popup menus
        AttributeLabelListener attrListener = new AttributeLabelListener(node);
        // displays attributes of the node
        displayAttributes(node, attrListener);
        // displays children of the node
        displayChildren(node, attrListener);
        // validates the editor panel
        this.getParent().getParent().validate();
    }

    /**
     * deals with events occured on the popup menu
     * 
     * @param e an event
     */
    public void actionPerformed(ActionEvent e) {

    }

    /**
     * a listener for node values edition
     */
    private abstract class NodeValueListener {

        /**
         * The node which value is edited
         */
        TreeElement treeElement;

        /**
         * a constructor
         * 
         * @param treeElement the node which value is edited
         */
        public NodeValueListener(TreeElement treeElement) {
            this.treeElement = treeElement;
        }
    }

    /**
     * a listener for string node values edition
     */
    private class StringFocusListener extends NodeValueListener implements
            FocusListener {

        /**
         * a constructor
         * 
         * @param treeElement the node which value is edited
         */
        public StringFocusListener(TreeElement treeElement) {
            super(treeElement);
        }

        /**
         * Invoked when a component gains the keyboard focus.
         * 
         * @param e the Focus event describing this change
         */
        public void focusGained(FocusEvent e) {

        }

        /**
         * Invoked when a component looses the keyboard focus.
         * 
         * @param e the Focus event describing this change
         */
        public void focusLost(FocusEvent e) {
            if (!e.isTemporary())
            // validates the change
                treeElement.nodeValueChanged();
        }

    }

    /**
     * a listener for string node values edition
     */
    private class StringNodeValueListener extends NodeValueListener implements
            DocumentListener {

        /**
         * a constructor
         * 
         * @param treeElement the node which value is edited
         */
        public StringNodeValueListener(TreeElement treeElement) {
            super(treeElement);
        }

        /**
         * called when text is inserted
         * 
         * @param e the Document event describing this change
         */
        public void insertUpdate(DocumentEvent e) {
            textChanged(e);
        }

        /**
         * validates a change in the text of a JTextComponent
         * 
         * @param e the Document event describing this change
         */
        private void textChanged(DocumentEvent e) {
            javax.swing.text.Document doc = e.getDocument();
            try {
                // changes the text
                treeElement.setNodeValue(doc.getText(0, doc.getLength()));
            } catch (BadLocationException ex) {
                // should never happend
            }
        }

        /**
         * called when text is removed
         * 
         * @param e the Document event describing this change
         */
        public void removeUpdate(DocumentEvent e) {
            textChanged(e);
        }

        /**
         * called when style is changed
         * 
         * @param e the Document event describing this change
         */
        public void changedUpdate(DocumentEvent e) {
            // we won't ever get this with a PlainDocument
        }
    }

    /**
     * a listener for string node values edition
     */
    private class ListNodeValueListener extends NodeValueListener implements
            ActionListener {

        /**
         * a constructor
         * 
         * @param treeElement the node which value is edited
         */
        public ListNodeValueListener(TreeElement treeElement) {
            super(treeElement);
        }

        /**
         * what to do if the selection in a comboBox changes
         */
        public void actionPerformed(ActionEvent e) {
            // gets the combo box
            JComboBox comboBox = (JComboBox) e.getSource();
            // gets old and new value
            String oldValue = treeElement.getNodeValue();
            String newValue = (String) comboBox.getSelectedItem();
            // changes the value
            treeElement.setNodeValue(newValue);
            // validates the change
            treeElement.nodeValueChanged();
            // makes the UndoManager aware of this action
            String name = treeElement.getNodeName();
            treeView.getEditor().getUndoManager().addEdit(
                    new UndoableAttributeEdition(
                            treeElement.getOwnerDocument(), name, oldValue,
                            newValue, (ElementImpl) ((Attr) treeElement)
                                    .getOwnerElement(), "change attribute "
                                    + name + "'s value"));
        }
    }

    /**
     * a listener for attribute's popup menus
     */
    private class AttributeLabelListener implements MouseListener {

        /**
         * the node this listener is listening at the attributes
         */
        TreeElement node;

        /**
         * constructor
         * 
         * @param node the attribute node this listener will listen at
         */
        public AttributeLabelListener(TreeElement node) {
            this.node = node;
        }

        /**
         * Invoked when the mouse has been clicked on a component.
         */
        public void mouseClicked(MouseEvent e) {
            // no action here
        }

        /**
         * Invoked when a mouse button has been pressed on a component.
         */
        // note that popup events are processed either in mousePressed
        // or in mouseReleased because it depends on the operating system
        public void mousePressed(MouseEvent e) {
            // only deals with popup events
            if (e.isPopupTrigger()) {
                showPopup(e);
            }
        }

        /**
         * Invoked when a mouse button has been released on a component.
         */
        // note that popup events are processed either in mousePressed
        // or in mouseReleased because it depends on the operating system
        public void mouseReleased(MouseEvent e) {
            // only deals with popup events
            if (e.isPopupTrigger()) {
                showPopup(e);
            }
        }

        /**
         * Invoked when the mouse enters a component.
         */
        public void mouseEntered(MouseEvent e) {
            // no action here
        }

        /**
         * Invoked when the mouse exits a component.
         */
        public void mouseExited(MouseEvent e) {
            // no action here
        }

        /**
         * this method displays, if needed, a popup menu adapted to the node
         * clicked
         * 
         * @param e the mouse event
         */
        public void showPopup(MouseEvent e) {
            JPopupMenu popupMenu = new JPopupMenu();
            JMenuItem removeItem = new JMenuItem(REMOVE);
            removeItem.addActionListener(new PopupListener(node, ((JLabel) e
                    .getSource()).getText()));
            popupMenu.add(removeItem);
            // shows the popup menu
            popupMenu.show(e.getComponent(), e.getX(), e.getY());
        }
    }

    /**
     * a listener for the popup menu
     */
    public class PopupListener implements ActionListener {

        /**
         * the node containing the attribute the popup was launched on
         */
        private TreeElement node;
        /**
         * the name of the attribute the popup was launched on
         */
        private String name;

        /**
         * constructor
         * 
         * @param node the Node on which the popup menu was launched
         */
        public PopupListener(TreeElement node, String name) {
            this.node = node;
            this.name = name;
        }

        /**
         * called each time an action occurs
         * 
         * @param e the action event
         */
        public void actionPerformed(ActionEvent e) {
            ((ElementImpl) node).removeAttribute(name, treeView);
        }
    }

}