/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview;

import com.simulogic.dataview.plots.AbstractPlot;
import com.simulogic.dataview.plots.ChannelAttribute;
import com.simulogic.dataview.plots.LinePlot;

/**
 * Class that represents one data set (channel)
 */
public class DataChannel {

    private String name = "";
    private String units = "";
    private double data[];
    private AbstractPlot plot;
    private ChannelAttribute attribute;

    private double mindata = Double.POSITIVE_INFINITY;
    private double maxdata = Double.NEGATIVE_INFINITY;

    /**
     * Default constructor
     */
    public DataChannel() {
        this("", 0);
    }

    /**
     * Creates new DataChannel
     * 
     * @param name name of the channel
     * @param recs number of records in the channel
     */
    public DataChannel(String name, int recs) {
        this.setName(name);
        this.data = new double[recs];
        this.attribute = new ChannelAttribute();
        this.plot = new LinePlot();
    }

    /**
     * Returns the smallest value in data array
     * 
     * @return lower limit
     */
    public double minLimit() {
        return mindata;
    }

    /**
     * Returns the largest value in data array
     * 
     * @return upper limit
     */
    public double maxLimit() {
        return maxdata;
    }

    /**
     * Returns name of this channel
     * 
     * @return channel name
     */
    public String getName() {
        return name;
    }

    /**
     * Sets name for this channel
     * 
     * @param name new channel name
     */
    public void setName(String name) {
        int beg = 0;
        int end = 0;
        beg = name.lastIndexOf("(");
        end = name.lastIndexOf(")");
        if (end > beg) this.units = name.substring(beg + 1, end);
        
        this.name = name;
    }

    /**
     * Returns units of this channel
     * 
     * @return channel units
     */
    public String getUnits() {
        return units;
    }

    /**
     * Sets units for this channel
     * 
     * @param units new channel units
     */
    public void setUnits(String units) {
        this.units = units;
    }

    /**
     * Returns channel plot
     * 
     * @return channel plot
     */
    public AbstractPlot getPlot() {
        return plot;
    }

    /**
     * Sets plot for this channel
     * 
     * @param plot new channel plot
     */
    public void setPlot(AbstractPlot plot) {
        this.plot = plot;
    }
    
    /**
     * Returns attribute of this channel
     * 
     * @return channel attribute
     */
    public ChannelAttribute getAttribute() {
        return attribute;
    }

    /**
     * Sets attribute for this channel
     * 
     * @param attribute new channel attribute
     */
    public void setAttribute(ChannelAttribute attribute) {
        this.attribute = attribute;
    }

    /**
     * Returns size of the data array
     * 
     * @return size of data array
     */
    public int size() {
        return this.data.length;
    }

    /**
     * Returns value for given index in data array
     * 
     * @param inc index in the array
     * @return data value
     * @throws ArrayIndexOutOfBoundsException when index is wrong
     */
    public double getData(int inc) throws ArrayIndexOutOfBoundsException {
        if (inc < this.data.length) {
            return this.data[inc];
        } else {
            throw (new ArrayIndexOutOfBoundsException(inc + ">"
                    + this.data.length));
        }
    }

    /**
     * Sets value at the given index in data array
     * 
     * @param inc index in the array
     * @param val new data value
     * @throws ArrayIndexOutOfBoundsException when index is wrong
     */
    public synchronized void setData(int inc, double val)
            throws ArrayIndexOutOfBoundsException {
        if (inc < this.data.length) {
            this.data[inc] = val;
            mindata = Math.min(val, mindata);
            maxdata = Math.max(val, maxdata);
        } else {
            throw (new ArrayIndexOutOfBoundsException(inc + ">"
                    + this.data.length));
        }
    }
    

    /**
     * Grows data array by a specific number of data points
     * 
     * @param inc increment for data array size
     */
    public synchronized void grow(int inc) {
    	int length = data.length;
   
        double incr[] = new double[length + inc];
        System.arraycopy(data, 0, incr, 0, length);
        this.data = incr;
    }

    /**
     * Returns the string (ASCII) representation of the data contained in this
     * channel
     * 
     * @return ASCII representation of the channel
     */
    public String toString() {
        StringBuffer datOut = new StringBuffer();
        datOut.append(this.getName());
        for (int k = 0; k < this.data.length; k++) {
            datOut.append(this.getData(k));
        }
        return datOut.toString();
    }

}